;;;;"test.scm" FreeSnell optics validation suite	-*-scheme-*-
;;; Copyright (C) 2003, 2004 Aubrey Jaffer

;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

;;; http://localhost/~jaffer/FreeSnell/validation

(require 'FreeSnell)
(require 'databases)
(require 'database-interpolate)

(define (round-from-table table column)
  (define get  (dbinterp:memoize (table 'get column) 3))
  (define prev (dbinterp:memoize (table 'isam-prev) 3))
  (define next (table 'isam-next))
  (lambda (x)
    (let ((nxt (next x)))
      (if nxt (set! nxt (car nxt)))
      (let ((prv (prev (or nxt x))))
	(if prv (set! prv (car prv)))
	(cond ((not nxt) (get prv))
	      ((not prv) (get nxt))
	      (else
	       (if (> (/ (- x prv) (- nxt prv)) .5)
		   (get nxt)
		   (get prv))))))))

;;; Reducing the size of plots to closer match the web plots.
;;;(define graph:dimensions '(512 256))
;;;(define graph:font-size 12)
;;; Output png files for validation webpage.
;;;(define *output-format* 'png)

(defvar nk (open-database (or (getenv "NK_DATABASE_PATH") "nk.rwb") 'rwb-isam))
;;(defvar nk (open-database "/home/jaffer/cool/nk-sopra.rwb" 'rwb-isam))
(defvar ge (interpolate-from-table (open-table nk 'ge) 2))
(defvar zns (interpolate-from-table (open-table nk 'znscub) 2))
(defvar tio2 (interpolate-from-table (open-table nk 'tio2) 2))
(defvar sio2 (interpolate-from-table (open-table nk 'sio2) 2))

(defvar au (interpolate-from-table (open-table nk 'au) 2))
(defvar ag (interpolate-from-table (open-table nk 'ag) 2))
(defvar al (interpolate-from-table (open-table nk 'al) 2))
;;;Glasses
(defvar *7059*
  (interpolate-from-table (open-table nk (string->symbol "7059")) 2))
(defvar BK7 (interpolate-from-table (open-table nk 'BK7) 2))
(defvar COR7059 (interpolate-from-table (open-table nk 'COR7059) 2))
(defvar LASF9 (interpolate-from-table (open-table nk 'LASF9) 2))
(defvar SF11 (interpolate-from-table (open-table nk 'SF11) 2))

;;;; Handy for comparing FreeSnell graphs to optics on the web
(define optics:tmp "/tmp/optics.url")
(define (browse-optics-url url)
  (define current-url (and (file-exists? optics:tmp)
			   (call-with-input-file optics:tmp read)))
  (cond ((equal? url current-url))
	(else (call-with-output-file optics:tmp
		(lambda (oport) (write url oport)))
	      (browse-url url))))

;; works
(define (wide-bp)
  (define ZnS 2.2)			;znscub +0.01i
  (define Ge 4.2)			;ge
  ;;(browse-optics-url "http://www.sspectra.com/designs/irbp.html")
  (plot-response
   (title "Wide Infrared Bandpass Filter" "wide-bp")
   (output-format 'png 512 256 12)
   (samples 300)
   (incident 0 'T)
   ;;(IR Ge 'real 'imag) (IR ZnS 'real 'imag)
   (linear 2e-6 7e-6)
   (optical-stack
    (substrate 1)
    (layer	ZnS   528.64e-9)
    (layer	Ge    178.96e-9)
    (layer	ZnS   250.12e-9)
    (layer	Ge    123.17e-9)
    (layer	ZnS   294.15e-9)
    (layer	Ge    156.86e-9)
    (layer	ZnS   265.60e-9)
    (layer	Ge    134.34e-9)
    (layer	ZnS   266.04e-9)
    (layer	Ge    147.63e-9)
    (layer	ZnS   289.60e-9)
    (layer	Ge    133.04e-9)
    (layer	ZnS   256.22e-9)
    (layer	Ge    165.16e-9)
    (layer	ZnS   307.19e-9)
    (layer	Ge    125.25e-9)
    (layer	ZnS   254.28e-9)
    (layer	Ge    150.14e-9)
    (layer	ZnS   168.55e-9)
    (layer	Ge     68.54e-9)
    (layer	ZnS   232.65e-9)
    (layer	Ge    125.48e-9)
    (layer	ZnS   238.01e-9)
    (layer	Ge    138.25e-9)
    (layer	ZnS   268.21e-9)
    (layer	Ge     98.28e-9)
    (layer	ZnS   133.58e-9)
    (layer	Ge    125.31e-9)
    (layer	ZnS   224.72e-9)
    (layer	Ge     40.79e-9)
    (layer	ZnS   564.95e-9)
    (layer	Ge    398.52e-9)
    (layer	ZnS   710.47e-9)
    (layer	Ge    360.01e-9)
    (layer	ZnS   724.86e-9)
    (layer	Ge    353.08e-9)
    (layer	ZnS   718.52e-9)
    (layer	Ge    358.23e-9)
    (layer	ZnS   709.26e-9)
    (layer	Ge    370.42e-9)
    (layer	ZnS   705.03e-9)
    (layer	Ge    382.28e-9)
    (layer	ZnS   720.06e-9)
    (layer	Ge    412.85e-9)
    (layer	ZnS   761.47e-9)
    (layer	Ge     48.60e-9)
    (layer	ZnS    97.33e-9)
    (substrate  4)		 ;sspectra.com claims 4 for substrate.
    )))

;;; Matches
(define (anti)
  (define H 4.2)			;Ge
  (define L 2.2)			;ZnS
  ;;(browse-optics-url "http://www.sspectra.com/designs/IR_AR.html")
  (plot-response
   (title "Growing an Antireflection Coating" "anti")
   (output-format 'png 600 256 12)
   (linear 7700e-9 12300e-9)
   (range 0 .02)
   (incident 0 'R)
   (optical-stack
    (substrate 1)
    (layer L   1076.97e-9)
    (layer H    629.03e-9)
    (layer L    457.49e-9)
    (layer H    131.55e-9)
    (layer L   1234.92e-9)
    (layer H   1142.59e-9)
    (layer L   2590.54e-9)
    (layer H     81.75e-9)
    (layer L   1827.50e-9)
    (layer H     84.60e-9)
    (layer L    563.34e-9)
    (layer H    285.97e-9)
    (layer L    157.14e-9)
    (substrate 4.0)			;Ge
    )))

;;; Matches
(define (immersed)
  (define SiO2 1.45)
  (define TiO2 2.35)
  (define BK7 1.5164)
  ;;(browse-optics-url "http://www.sspectra.com/designs/immersedWPol.html")
  (plot-response
   (title "Immersed Wide-Angle Polarizer" "immersed")
   (output-format 'png 560 250 12)
   (samples 50)
   (range .9 1)
   (wavelength .653e-6 'T_p)
   (wavelength .643e-6 'T_p)
   (wavelength .633e-6 'T_p)
   (wavelength .623e-6 'T_p)
   (wavelength .613e-6 'T_p)
   (angles 51 71)
   (optical-stack
    (substrate BK7)
    (layer TIO2      83.62e-9)
    (layer SIO2      74.91e-9)
    (layer TIO2      94.86e-9)
    (layer SIO2     119.09e-9)
    (layer TIO2      90.21e-9)
    (layer SIO2     129.48e-9)
    (layer TIO2      81.87e-9)
    (layer SIO2     123.04e-9)
    (layer TIO2      84.03e-9)
    (layer SIO2     138.23e-9)
    (layer TIO2      81.34e-9)
    (layer SIO2     131.15e-9)
    (layer TIO2      83.85e-9)
    (layer SIO2     152.21e-9)
    (layer TIO2      80.48e-9)
    (layer SIO2     126.78e-9)
    (layer TIO2      59.95e-9)
    (substrate BK7)
    )))

;;; This gives performance as described.
(define (prism)
  (define H 2.35)
  (define L 1.45)
  (let ((stack
	 (optical-stack
	  (nominal  .55e-6)
	  (substrate  1.85)
	  (layer	    H     11.16e-9)
	  (layer	    L     48.87e-9)
	  (layer	    H     35.94e-9)
	  (layer	    L     68.58e-9)
	  (layer	    H     39.50e-9)
	  (layer	    L     79.77e-9)
	  (layer	    H     46.20e-9)
	  (layer	    L     96.61e-9)
	  (layer	    H     49.71e-9)
	  (layer	    L    102.74e-9)
	  (layer	    H     50.49e-9)
	  (layer	    L    102.74e-9)
	  (layer	    H     49.71e-9)
	  (layer	    L     96.61e-9)
	  (layer	    H     46.20e-9)
	  (layer	    L     79.77e-9)
	  (layer	    H     39.50e-9)
	  (layer	    L     68.58e-9)
	  (layer	    H     35.94e-9)
	  (layer	    L     48.87e-9)
	  (layer	    H     11.16e-9)
	  (substrate  1.85))))
    ;;(browse-optics-url "http://www.sspectra.com/designs/ftr_polar.html")
    (plot-response (title "New Thin Film Polarizing Beamsplitter - S" "prism-s")
		   (range .99 1)
		   (output-format 'png 560 250 12)
		   (samples 50)
		   (linear .4e-6 .7e-6)
		   (incident 65 'T_s)
		   (incident 66 'T_s)
		   (incident 67 'T_s)
		   (incident 68 'T_s)
		   (incident 69 'T_s)
		   (incident 70 'T_s)
		   (incident 71 'T_s)
		   (incident 72 'T_s)
		   (incident 73 'T_s)
		   (incident 74 'T_s)
		   (incident 75 'T_s)
		   stack)
    (plot-response (title "New Thin Film Polarizing Beamsplitter - P" "prism-p")
		   (range -200 -10 'db)
		   (output-format 'png 560 250 12)
		   (samples 10)
		   (linear .4e-6 .7e-6)
		   (incident 65 'T_p)
		   (incident 66 'T_p)
		   (incident 67 'T_p)
		   (incident 68 'T_p)
		   (incident 69 'T_p)
		   (incident 70 'T_p)
		   (incident 71 'T_p)
		   (incident 72 'T_p)
		   (incident 73 'T_p)
		   (incident 74 'T_p)
		   (incident 75 'T_p)
		   stack)))

;;;Sh. A. Furman and A. V. Tikhonravov.
;;;Basics of Optics of Multilayer Systems.
;;;Editions Fronti`eres, Gif-sur-Yvette Cedex -- France, 1992.
;;;http://www.optilayer.com/cgi-bin/od/odn.pl?ID=2
;;
;;Fig. 2.2: Reflectances of the 6-layer antireflection coatings
;;with nS = 1.52, nH = 2.30, nL = 1.45 (lambda = 500nm)
;; (1)6S .318H .34L 1.977H .106L .375H 1.099L,
;; (2)6S .34H .391L 1.104H .188L .710H 1.216L
(define (fig.2.2)
  (define nH 2.30)
  (define nL 1.45)
  (define nS 1.52)
  (define common (list (output-format 'png 625 315 12)
		       (incident 0 'R)
		       (linear 400e-9 900e-9)
		       (range 0 .02)))
  (apply plot-response
	 (title "fig-2.2 1" "fig-2.2-1")
	 (optical-stack
	  (nominal (/ 500e-9 4))
	  (substrate 1)
	  (layer nL 1.099)
	  (layer nH 0.375)
	  (layer nL 0.106)
	  (layer nH 1.977)
	  (layer nL 0.34)
	  (layer nH 0.318)
	  (substrate nS))
	 common)
  (apply plot-response
	 (title "fig-2.2 2" "fig-2.2-2")
	 (optical-stack
	  (nominal (/ 500e-9 4))
	  (substrate 1)
	  (layer nL 1.216)
	  (layer nH 0.710)
	  (layer nL 0.188)
	  (layer nH 1.104)
	  (layer nL 0.391)
	  (layer nH 0.34)
	  (substrate nS))
	 common))

;;Fig. 2.5: Reflectance of the 8-layer antireflection coating
;;with nS = 1.52, nH = 2.30, nL = 1.38 (lambda = 500nm)
;;  8S .295H .291L .149H .074L 1.915H ?? .107H 1.052L
;;The geometric thicknesses of the layers in nanometers
;;(starting from the substrate) are equal to
;;14.1; 26.4; 8.1; 6.7; 104.1; 9.7; 16.9; 95.3.
;;The optical thicknesses of the layers are equal to
;;32.4; 36.4; 18.6; 9.3; 239.4; 13.4; 38.9; 131.5.
(define (fig.2.5)
  (define nH 2.30)
  (define nL 1.38)
  (define nS 1.52)
  (plot-response (title "fig-2.5" "fig-2.5")
		 (output-format 'png 695 325 12)
		 (incident 0 'R)
		 (linear 400e-9 800e-9)
		 (range 0 .004)
		 (optical-stack
		  (nominal 500e-9)
		  (substrate 1)
		  (layer nL 95.3e-9)
		  (layer nH 16.9e-9)
		  (layer nL 9.7e-9)
		  (layer nH 104.1e-9)
		  (layer nL 6.7e-9)
		  (layer nH 8.1e-9)
		  (layer nL 26.4e-9)
		  (layer nH 14.1e-9)
		  (substrate nS))))

;;; works
(define (HL7)
  (define glass 1.52)
  (define SiO2 1.46)
  (define TiO2 2.40)
  ;;(browse-optics-url "http://www.kruschwitz.com/HR%27s.htm#The Basic HR")
  (plot-response
   (title "(HL)^7H on glass" "hl7")
   (output-format 'png 400 175 12)
   ;;(range -10 0 'db)
   ;;(db)
   ;;(linear 0 (* 2 550e-9))
   (range .99 1.0)
   (linear 400e-9 700e-9)
   (incident 0 'R)
   (optical-stack
    (nominal  550e-9)
    (substrate 1)
    (layer TiO2 1/4)
    (layer SiO2 1/4)
    (layer TiO2 1/4)
    (layer SiO2 1/4)
    (layer TiO2 1/4)
    (layer SiO2 1/4)
    (layer TiO2 1/4)
    (layer SiO2 1/4)
    (layer TiO2 1/4)
    (layer SiO2 1/4)
    (layer TiO2 1/4)
    (layer SiO2 1/4)
    (layer TiO2 1/4)
    (layer SiO2 1/4)
    (layer TiO2 1/4)
    (substrate glass)
    )))

(define (cold-mirror)
  (define glass 1.52)
  (define SiO2 1.46)
  (define TiO2 2.40)
  (define nom 550e-9)
  ;;(browse-optics-url "http://www.kruschwitz.com/Cold/hot.htm")
  (plot-response
   (title "Cold Mirror" "cold-mirror")
   (output-format 'png 435 175 12)
   (linear 300e-9 1100e-9)
   (incident 0 'R)
   (optical-stack (substrate 1)
		  (nominal nom)
		  (layer SiO2 1/2)
		  (nominal (* 0.8 nom))
		  (repeat 5
			  (layer TiO2 1/8)
			  (layer SiO2 1/4)
			  (layer TiO2 1/8))
		  (nominal (* 1.0 nom))
		  (repeat 5
			  (layer TiO2 1/8)
			  (layer SiO2 1/4)
			  (layer TiO2 1/8))
		  (nominal (* 1.2 nom))
		  (repeat 5
			  (layer TiO2 1/8)
			  (layer SiO2 1/4)
			  (layer TiO2 1/8))
		  (substrate glass))))

(define (cold-mirrors)
  (define glass 1.52)
  (define SiO2 1.46)
  (define TiO2 2.40)
  ;;(browse-optics-url "http://www.sspectra.com/designs/coldmirror.html")  
  ;;"http://www.sspectra.com/designs/coldmirror2.gif"
  (plot-response
   (title "Cold Mirrors" "cold-mirror1")
   (output-format 'png 585 280 12)
   (range 0 1)
   (linear 380e-9 1000e-9)
   (incident 0 'R)
   (optical-stack
    (layer TIO2      34.40e-9)
    (layer SIO2     158.61e-9)
    (layer TIO2      53.24e-9)
    (layer SIO2     128.87e-9)
    (layer TIO2      67.77e-9)
    (layer SIO2     102.19e-9)
    (layer TIO2      79.13e-9)
    (layer SIO2      97.19e-9)
    (layer TIO2      53.07e-9)
    (layer SIO2     162.54e-9)
    (layer TIO2      36.16e-9)
    (layer SIO2      66.56e-9)
    (layer TIO2      46.48e-9)
    (layer SIO2      87.18e-9)
    (layer TIO2      48.92e-9)
    (layer SIO2      82.52e-9)
    (layer TIO2      42.56e-9)
    (layer SIO2      73.13e-9)
    (layer TIO2      43.97e-9)
    (layer SIO2     102.57e-9)
    (layer TIO2      44.21e-9)
    (substrate glass)))
  (plot-response
   (title "Cold Mirrors" "cold-mirror2")
   (output-format 'png 585 280 12)
   (range 0 1)
   (linear 380e-9 2000e-9)
   (incident 0 'R)
   (optical-stack
    (layer TIO2    19.03e-9)
    (layer SIO2   191.13e-9)
    (layer TIO2    51.30e-9)
    (layer SIO2   137.38e-9)
    (layer TIO2    61.86e-9)
    (layer SIO2    99.64e-9)
    (layer TIO2    81.48e-9)
    (layer SIO2   102.08e-9)
    (layer TIO2    53.15e-9)
    (layer SIO2   162.37e-9)
    (layer TIO2    36.85e-9)
    (layer SIO2    70.41e-9)
    (layer TIO2    39.55e-9)
    (layer SIO2    88.77e-9)
    (layer TIO2    51.97e-9)
    (layer SIO2    86.41e-9)
    (layer TIO2    51.05e-9)
    (layer SIO2    66.36e-9)
    (layer TIO2    28.39e-9)
    (layer SIO2   124.04e-9)
    (layer TIO2    33.76e-9)
    (substrate glass))))

;;; works
(define (reflector-50)
  (define L 1.45)
  (define H 2.35)
  ;;(browse-optics-url "http://www.sspectra.com/designs/bbhr.html")
  (plot-response
   (title "Broadband High-Reflection Coating at 50 Degrees" "reflect-50")
   (output-format 'png 560 256 12)
   (range .8 1)
   (linear .4e-6 .8e-6) (incident 50 'R_p) ;(incident 40 'R_p)
   ;;(wavelength .6e-6 'R_p) (wavelength .7e-6 'R_p) (angles 0 90)
   (optical-stack
    (nominal  .6e-6)
    (substrate 1)
    (layer H      49.35e-9)
    (layer L      80.64e-9)
    (layer H      45.96e-9)
    (layer L      91.80e-9)
    (layer H      51.59e-9)
    (layer L      86.62e-9)
    (layer H      46.16e-9)
    (layer L      90.92e-9)
    (layer H      53.49e-9)
    (layer L      94.22e-9)
    (layer H      46.62e-9)
    (layer L      82.60e-9)
    (layer H      52.26e-9)
    (layer L     101.16e-9)
    (layer H      56.86e-9)
    (layer L     101.69e-9)
    (layer H      63.51e-9)
    (layer L     121.69e-9)
    (layer H      61.47e-9)
    (layer L     111.11e-9)
    (layer H      63.21e-9)
    (layer L     121.61e-9)
    (layer H      71.53e-9)
    (layer L     113.89e-9)
    (layer H      55.42e-9)
    (layer L     104.04e-9)
    (layer H      69.96e-9)
    (layer L     137.71e-9)
    (layer H      71.17e-9)
    (layer L     141.89e-9)
    (layer H      82.96e-9)
    (layer L     141.64e-9)
    (layer H      85.36e-9)
    (layer L     129.75e-9)
    (layer H      72.02e-9)
    (layer L     178.45e-9)
    (layer H      83.10e-9)
    (layer L     154.04e-9)
    (layer H      74.35e-9)
    (layer L     152.15e-9)
    (layer H      80.56e-9)
    (layer L     171.35e-9)
    (layer H      97.87e-9)
    (substrate 1.52)
    )))

;;; good match midband; not so good elsewhere.
(define (omni-mirror)
  (define H 4.6)			;tellurium
  (define L 1.6)			;polystyrene
  ;;(browse-optics-url "http://www.sspectra.com/designs/omnirefl.html")
  (plot-response
   (title "Dielectric Omnidirectional Reflector" "omni-mirror")
   (output-format 'png 565 250 12)
   (range 0 1)
   (linear 6e-6 20e-6)
   (incident 10 'R)
   (incident 20 'R)
   (incident 30 'R)
   (incident 40 'R)
   (incident 50 'R)
   (incident 60 'R)
   (incident 70 'R)
   (incident 80 'R)
   (incident 89 'R)
   (optical-stack
    (substrate 1)
    (layer	    H    399.00e-9)
    (layer	    L   2165.33e-9)
    (layer	    H    696.76e-9)
    (layer	    L   2097.40e-9)
    (layer	    H    701.51e-9)
    (layer	    L   2089.58e-9)
    (layer	    H    700.83e-9)
    (layer	    L   2112.33e-9)
    (layer	    H    691.88e-9)
    (substrate  1.5442)
    )))

;;; Matches at 0.o and 45.o
(define (protected-al)
  ;;(browse-optics-url "http://www.kruschwitz.com/HR%27s.htm")
  (plot-response
   (title "Protected Aluminum Mirror#Protected Metal" "protected-al")
   (output-format 'png 440 155 12)
   (range .7 .95)
   (incident 0 'R)
   (incident 45 'R)
   ;;(IR AL 'real 'imag)
   (linear .3e-6 .9e-6)
   ;;(logarithmic .35e-6 16e-6)
   (optical-stack
    ;;(layer     2       (/ 1.65 4))
    (layer     1.45      (/ 1.75 4) .6e-6)
    (layer     AL        5e-6)
    )))

(define (Si2O3-Al)
  (define Si2O3 1.65)
  (define SrF2 1.4)
  ;;(browse-optics-url "http://www.mellesgriot.com/products/optics/oc_5_1.htm")
  (plot-response
   (title "Si2O3 Protected Aluminum (/011)" "Si203-Al")
   (output-format 'png 395 205 12)
   (range .7 1)
   (incident 0 'R)
   (incident 45 'R_s 'R_p)
   ;;(IR AL 'real 'imag)
   (linear 400e-9 750e-9)
   (optical-stack
    (nominal 550e-9)
    (layer     Si2O3      1/2)		;Si2O3
    (layer     AL        5e-6)
    )))

;;; Matches for one, two, and three HL pairs.
(define (enhanced-al)
  ;;... by increasing the number of periods, the
  ;;reflectivity increases, but the high reflectivity region narrows.
  (define H 2.40)
  (define L 1.46)
  ;;(browse-optics-url "http://www.kruschwitz.com/HR%27s.htm#Enhanced Metal")
  (plot-response
   (title "Enhanced Aluminum Mirror" "enhanced-al")
   (output-format 'png 435 155 12)
   (range .6 1)
   (linear .3e-6 .9e-6)
   (incident 0 'R)
   (optical-stack
    (nominal  550e-9)
    (layer     H         1/4)
    (layer     L         1/4)
    (layer     H         1/4)
    (layer     L         1/4)
    (layer     H         1/4)
    (layer     L         1/4)
    (layer     AL        5e-6)
    )))

(define (al-mirror)
  ;;(browse-optics-url "http://www.mellesgriot.com/products/optics/oc_5_1.htm")
  (plot-response
   (title "Aluminum Mirror (/016)" "Al-mirror")
   (output-format 'png 390 200 12)
   (range .75 1)
   (incident 0 'R)
   (linear .4e-6 1.1e-6)
   (optical-stack (layer Al 5e-6))))

(define (zal-mirror)
  (define al (round-from-table (open-table nk 'al) 2))
  ;;(browse-optics-url "http://www.mellesgriot.com/products/optics/oc_5_1.htm")
  (plot-response
   (title "Aluminum Mirror" "zAl-mirror")
   (output-format 'png 440 225 12)
   (incident 0 'R)
   (linear .75e-6 .925e-6)
   (IR Al 'n 'k 'ec)
   (optical-stack (layer Al 5e-6))))

(define (bare-au)
  ;;(define au (round-from-table (open-table nk 'au) 2))
  ;;(browse-optics-url "http://www.mellesgriot.com/products/optics/oc_5_1.htm")
  (plot-response
   (title "Bare Gold (/45)" "Au-mirror")
   (output-format 'png 390 200 12)
   (range 0 1)
   (incident 0 'R)
   (linear .4e-6 2.8e-6)
   ;;(IR Au 'n 'ec)
   (optical-stack (layer Au .1e-6))))

;;;; From US patent 4,337,990 issued May. 1976 to Fan et al.
;;; 359/360. Transparent heat-mirror
;;; Poor match.  TiO2 has bad UV kink
(define (hot-mirror-2)
  ;;(define TiO2 2.35)
  ;;(browse-optics-url "http://patft.uspto.gov/netacgi/nph-Parser?Sect1=PTO1&Sect2=HITOFF&d=PALL&p=1&u=/netahtml/srchnum.htm&r=1&f=G&l=50&s1=4,337,990.WKU.&OS=PN/4,337,990&RS=PN/4,337,990")
  (plot-response (title "Hot Mirror 2" "hot-2")
		 (output-format 'png 640 365 12)
		 (incident 0 'R 'T)
		 (incident 40 'T)
		 (logarithmic 0.3e-6 10e-6)
		 ;;(logarithmic 0.5e-6 2e-6)
		 (optical-stack
		  (layer TiO2 18e-9)
		  (layer Ag   18e-9)
		  (layer TiO2 18e-9)
		  ;;(layer *7059* 1e-3)
		  (substrate *7059*))
		 (IR TiO2 'real 'ec)))

;; has slight frequency shift
(define (metal-bp)
  ;;(define AU 1.693+1.883i)
  (define BK7 1.5164)
  (define SiO2 1.45)
  (define TiO2 2.35)
  ;;(browse-optics-url "http://www.sspectra.com/designs/mdbp.html")
  (plot-response
   (title "Metal-Dielectric Bandpass Filter" "metal-bp")
   (output-format 'png 560 215 12)
   (incident 0 'T 'A)
   ;;(IR AU 'real 'imag)
   (range 0 1)
   (linear .7e-6 1e-6)
   (marker 8.70e-7)
   (optical-stack
    (nominal  870e-9)
    (substrate 1)
    (layer     TIO2     101.94e-9)
    (layer     AU        24.97e-9)
    (layer     SIO2     127.87e-9)
    (layer     TIO2      94.24e-9)
    (layer     SIO2     302.57e-9)
    (layer     TIO2      94.24e-9)
    (layer     SIO2     121.94e-9)
    (layer     AU        23.33e-9)
    (substrate  BK7)
    )))

;; has slight frequency shift
(define (dual-bp)
  ;;(define AU 1.693+1.883i)
  (define BK7 1.5164)
  (define SiO2 1.45)
  (define TiO2 2.35)
  ;;(browse-optics-url "http://www.sspectra.com/designs/mdbp2.html")
  (plot-response
   (title "Metal-Dielectric Dual Bandpass Filter" "dual-bp")
   (output-format 'png 560 215 12)
   (range 0 1)
   (incident  0 'T)			; 'A
   (linear .7e-6 1e-6)
   (marker .770e-6 .920e-6)
   (optical-stack
    (nominal .920e-6)
    (substrate 1)
    (layer  AU        10.76e-9)
    (layer  SIO2     102.01e-9)
    (layer  TIO2      95.25e-9)
    (layer  SIO2      10.49e-9)
    (layer  AU         4.16e-9)
    (layer  SIO2     270.66e-9)
    (layer  TIO2      93.69e-9)
    (layer  SIO2     152.40e-9)
    (layer  TIO2      39.24e-9)
    (layer  SIO2      49.67e-9)
    (layer  TIO2      96.33e-9)
    (layer  SIO2     115.39e-9)
    (layer  AU        16.58e-9)
    (substrate  BK7)
    )))

(define (polymer-ag)
  (define (metal w) (granular-IR (ag w) 1/6 1.5))
  (define (metal/2 w) (granular-IR (ag w) 1/12 1.5))
  (define thk 154e-9)
  (define trns 0.1)
  ;;(browse-optics-url "http://web.mit.edu/cmse/www/IRG-II-2.nug02.html")
  (plot-response
   (title "Polymer-Ag" "poly-Ag")
   (output-format 'png 650 250 12)
   (IR metal 'real 'ec)
   (IR metal/2 'real 'ec)
   (color 0 'R)
   (color 30 'R)
   (incident 30 'R)
   (linear 300e-9 1500e-9)
   (range 0 1)
   (optical-stack (nominal 1.03e-6)
		  (repeat 4
			  (layer metal/2  (* trns thk))
			  (layer metal (* (- 1 (* 2 trns)) thk))
			  (layer metal/2  (* 2 trns thk))
			  (layer 1.5   (* (- 1 (* 2 trns)) thk))
			  (layer metal/2  (* trns thk))
			  )
		  (layer metal/2  (* trns thk))
		  (layer metal (* (- 1 (* 2 trns)) thk))
		  (layer metal/2  (* 2 trns thk))
		  (substrate BK7))))

(define (AgMgO)
  (define MgO 1.72)
  (define (metal w) (granular-IR (Ag w) 0.67 1))
  ;;(browse-optics-url "http://www.phys.ntnu.no/~ingves/Software/GranularFilm/")
  (plot-response
   (title "3.0.nm Ag deposit on an MgO substrate" "AgMgO-3.0")
   (output-format 'png 625 400 16)
   (range -.1 .35)
   (eV 1 4.5)
   (incident 45 'R_p)
   (optical-stack (layer metal 3e-9)
		  (substrate MgO)))
  (plot-response
   (title "4.5.nm Ag deposit on an MgO substrate" "AgMgO-4.5")
   (output-format 'png 625 400 16)
   (range -.1 .35)
   (eV 1 4.5)
   (incident 45 'R_p)
   (optical-stack (layer metal (/ 3e-9 0.67))
		  (substrate MgO))))

(define (Ge-trans)
  (define (ge/2 w) (* .5 (+ 1 (ge w))))
  (plot-response (title "Ge" "Ge-trans")
		 (incident 0 'T)
		 (logarithmic 0.7e-6 4e-6)
		 (range 0 1)
		 ;;(smooth)
		 (optical-stack (layer Ge  2.6e-5)
				(substrate BK7))
		 ))

(define (au-n)
  (plot-response (title "Au" "au-n")
		 (linear 450e-9 700e-9)
		 (ir au 'n)
		 (ir (lambda (w) (granular-IR (au w) 1.0 1)) 'n)
		 (ir (lambda (w) (granular-IR (au w) 0.9 1)) 'n)
		 (ir (lambda (w) (granular-IR (au w) 0.8 1)) 'n)
		 (ir (lambda (w) (granular-IR (au w) 0.7 1)) 'n)
		 (ir (lambda (w) (granular-IR (au w) 0.6 1)) 'n)
		 (ir (lambda (w) (granular-IR (au w) 0.5 1)) 'n)
		 (optical-stack (layer au 88e-10))
		 (substrate SiO2)
		 ))

(define (au-k)
  (plot-response (title "Au" "au-k")
		 (linear 450e-9 700e-9)
		 (ir au 'k)
		 (ir (lambda (w) (granular-IR (au w) 1.0 1)) 'k)
		 (ir (lambda (w) (granular-IR (au w) 0.9 1)) 'k)
		 (ir (lambda (w) (granular-IR (au w) 0.8 1)) 'k)
		 (ir (lambda (w) (granular-IR (au w) 0.7 1)) 'k)
		 (ir (lambda (w) (granular-IR (au w) 0.6 1)) 'k)
		 (ir (lambda (w) (granular-IR (au w) 0.5 1)) 'k)
		 (optical-stack (layer au 88e-10))
		 (substrate SiO2)
		 ))

(define (tests)
  (wide-bp)
  (anti)
  (immersed)
  (prism)
  (fig.2.2)
  (fig.2.5)
  (HL7)
  (cold-mirror)
  (cold-mirrors)
  (reflector-50)
  (omni-mirror)
  (protected-al)
  (Si2O3-Al)
  (hot-mirror-2)
  (al-mirror)
  (zal-mirror)
  (enhanced-al)
  (metal-bp)
  (dual-bp)
  (polymer-ag)
  (AgMgO)
  )

;;(tests)

(define (charts str illu)
  ;;(define BK7 1.5164)
  (write-color-table.png (string-append "Ag-" str) Ag 1e-9 100e-9 20 BK7 30 illu)
  (write-color-table.png (string-append "Au-" str) Au 1e-9 100e-9 20 BK7 30 illu))

(define (make-charts)
  (charts "D65" CIE:SI-D65)
  (charts "A" CIE:SI-A))
